from cameo_claw import it_distinct, it_download, it_group
import time
from cameo_claw.log import log


def get_lst_url():
    lst_device_id = [
        "11144353041",
        "11135655524",
        "11146220817",
        "11134915729",
        "11144949700",
        "11145345031",
        "11144535597",
        "11150348864",
        "11152332479",
        "11137445267",
        "11142706581",
        "11139427726",
        "11132757143",
        "11143315941",
        "11145428733",
        "11138263064",
        "11149040829",
        "11135424517",
        "11147350591",
        "11132265371",
        "11136533184",
        "11141843507",
        "11136480034",
        "11149117865",
        "11148832575",
        "11133512028",
        "11143988456",
        "11143119216",
        "11134364858",
        "11143240238",
        "11132477607",
        "11146482455",
        "11131367272",
        "11151053715",
        "11138740618",
        "11143570743",
        "11132083287",
        "11135577162",
        "11144277295",
        "11134472620",
        "11151283365",
        "11147041406",
        "11131824250",
        "11136322895",
        "11149357462",
        "11131209112",
        "11139221095",
        "11148103547",
        "11145230386",
        "11132685693",
    ]
    lst_url = []
    for device_id in lst_device_id:
        for i in range(1, 32):
            day = f'2022-03-{i:02}'
            url = f'https://iot.epa.gov.tw/fapi_open/topic-device-daily/topic_save.industry.rawdata.material/device_{device_id}/device_{device_id}_daily_{day}.csv.gz'
            lst_url.append(url)
    return lst_url


# def test_filter():
#     lst_select_column = ['createTime', 'deviceId', 'lat', 'localTime', 'lon', 'sensorId', 'value']
#     lst_distinct_column = ['deviceId', 'localTime', 'sensorId']
#     filter_column = 'sensorId'
#     filter_value = 'pm2_5'
#     target_directory = './data/parquet/'
#     multiprocessing_download_to_parquet(
#         get_lst_url(),
#         lst_select_column,
#         lst_distinct_column,
#         filter_column,
#         filter_value,
#         target_directory)

def test_it_download():
    target_directory = './data/topic_download/'
    int_total = len(get_lst_url())
    for int_progress, done_url in it_download(get_lst_url(), target_directory):
        print(f'int_progress/int_total = {int_progress}/{int_total}')
        print(f'done url: {done_url}')


def test_it_distinct():
    target_directory = './data/topic_distinct/'
    int_total = len(get_lst_url())
    lst_distinct_column = ['deviceId', 'localTime', 'sensorId']
    for int_progress, done_url in it_distinct(get_lst_url(), target_directory, lst_distinct_column):
        print(f'int_progress/int_total = {int_progress}/{int_total}')
        print(f'done url: {done_url}')


def test_it_group():
    target_directory = './data/topic_group/'
    int_total = len(get_lst_url())
    lst_distinct_column = ['deviceId', 'localTime', 'sensorId']
    lst_group_by_column = ['deviceId', 'sensorId']
    for int_progress, done_url in it_group(get_lst_url(), target_directory, lst_distinct_column, lst_group_by_column):
        print(f'int_progress/int_total = {int_progress}/{int_total}')
        print(f'done url: {done_url}')


if __name__ == '__main__':
    # t = time.time()
    # test_it_download()
    # log(f'test_it_download:{time.time() - t}')
    #
    # t = time.time()
    # test_it_distinct()
    # log(f'test_it_distinct:{time.time() - t}')

    t = time.time()
    test_it_group()
    log(f'test_it_group:{time.time() - t}')

# sh/go.sh



import sys
import yaml
from cameo_claw import it_distinct, it_download, it_group

help_message = '''
🐲 001 Install cameo_claw package
python3 -m pip install cameo_claw

🍇 002 Multiprocessing download, distinct rows, group by columns:
python3 -m cameo_claw '
function: it_group
lst_url:
  - https://cameo-claw-data.vercel.app/device_21152332479_daily_2022-03-30.csv
  - https://cameo-claw-data.vercel.app/device_21152332479_daily_2022-03-31.csv
target_directory: ./data/topic_group/
lst_distinct_column:
  - deviceId
  - localTime
  - sensorId
lst_group_by_column:
  - deviceId
  - sensorId'

🐍 003 cameo_claw coding with python
from cameo_claw import it_group
lst_url=[
'https://cameo-claw-data.vercel.app/device_21152332479_daily_2022-03-30.csv',
'https://cameo-claw-data.vercel.app/device_21152332479_daily_2022-03-31.csv']
int_total=len(lst_url)
target_directory = './data/topic_group/'
lst_distinct_column = ['deviceId', 'localTime', 'sensorId']
lst_group_by_column = ['deviceId', 'sensorId']
for int_progress, done_url in it_group(lst_url, target_directory, lst_distinct_column, lst_group_by_column):
    print(f'int_progress/int_total = {int_progress}/{int_total}')
    print(f'done url: {done_url}')
    
'''


def show_help():
    if len(sys.argv) == 1:
        print(help_message)
    if len(sys.argv) == 2:
        d = yaml.safe_load(sys.argv[1])
        if d['function'] == 'it_download':
            for int_progress, url in it_download(d['lst_url'], d['target_directory']):
                print(f'''{int_progress}/{len(d['lst_url'])}''', url)
        if d['function'] == 'it_distinct':
            for int_progress, url in it_distinct(d['lst_url'], d['target_directory'], d['lst_distinct_column']):
                print(f'''{int_progress}/{len(d['lst_url'])}''', url)
        if d['function'] == 'it_group':
            for int_progress, url in it_group(d['lst_url'], d['target_directory'], d['lst_distinct_column'],
                                              d['lst_group_by_column']):
                print(f'''{int_progress}/{len(d['lst_url'])}''', url)


if __name__ == '__main__':
    show_help()



from io import BytesIO
import os
import polars as pl
import gzip
from cameo_claw.functional import it_mp_f
from cameo_claw.net import requests_get


def a_distinct(url, target_directory, lst_distinct_column):
    def write(bytes1):
        df = pl.read_csv(bytes1)
        df = df.distinct(subset=lst_distinct_column)
        bytesio = BytesIO()
        df.to_csv(bytesio)
        filename = os.path.basename(url)
        filename = filename[:filename.find('.')]
        path = f'{target_directory}{filename}.csv.gz'
        with gzip.open(path, 'wb') as f:
            f.write(bytesio.getvalue())
        return url

    return requests_get(write, url, target_directory)


def it_distinct(lst_url, target_directory, lst_distinct_column):
    return it_mp_f(a_distinct, [tuple([url, target_directory, lst_distinct_column]) for url in lst_url])


def a_download(url, target_directory):
    def write(bytes1):
        filename = os.path.basename(url)
        path = f'{target_directory}{filename}'
        with open(path, 'wb') as f:
            f.write(bytes1)
        return url

    return requests_get(write, url, target_directory)


def it_download(lst_url, target_directory):
    return it_mp_f(a_download, [tuple([url, target_directory]) for url in lst_url])


def a_group(url, target_directory, lst_distinct_column, lst_group_by_column):
    def write(bytes1):
        df = pl.read_csv(bytes1)
        df = df.distinct(subset=lst_distinct_column)
        g = df.groupby(lst_group_by_column)
        for df in g:
            filename_tail = '_group_' + '-'.join(list(
                map(lambda column:
                    str(df.row(0)[df.find_idx_by_name(column)]).replace('_', '-'),
                    lst_group_by_column)))
            bytesio = BytesIO()
            df.to_csv(bytesio)
            filename = os.path.basename(url)
            filename = filename[:filename.find('.')]
            path = f'{target_directory}{filename}{filename_tail}.csv.gz'
            with gzip.open(path, 'wb') as f:
                f.write(bytesio.getvalue())
        return url

    return requests_get(write, url, target_directory)


def it_group(lst_url, target_directory, lst_distinct_column, lst_group_by_column):
    return it_mp_f(a_group,
                   [tuple([url, target_directory, lst_distinct_column, lst_group_by_column]) for url in lst_url])



from setuptools import setup

setup(
    name='cameo_claw',
    version='0.5.2',
    description='Multiprocessing download, distinct, group by. ⚡️FAST⚡️'
                '2022-04-05 v0.5.2 fix write_csv -> to_csv for python 3.7 bugs'
                '2022-04-05 v0.5.1 fix istarmap for python 3.7 bugs',
    release_notes='2022-04-05 v0.5.0 group by dataframe, ready for developers'
                  '2022-04-03 v0.4.0 add parameters'
                  '2022-04-03 v0.3.0 independent test'
                  '2022-04-02 v0.2.0 multiprocessing, download select to parquet'
                  '  . polars select and to parquet'
                  '  . 51.8s, ADSL, total 44MB parquets, convert to 1533 .parquet files'
                  '  . from 453MB .csv.gz, from 2.6GB .csv'
                  '2022-04-02 v0.1.1 default processes 25'
                  '2022-04-02 v0.1.0 initial',
    url='https://github.com/bohachu/cameo_claw',
    author='Bowen Chiu',
    author_email='bohachu@gmail.com',
    license='BSD 2-clause',
    packages=['cameo_claw'],
    install_requires=[
        'requests',
        'polars',
        'pyyaml',
        'tqdm'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
    ],
)



import multiprocessing.pool as mpp
import sys


def istarmap37(self, func, iterable, chunksize=1):
    """starmap-version of imap
    """
    if self._state != mpp.RUN:
        raise ValueError("Pool not running")
    if chunksize < 1:
        raise ValueError(
            "Chunksize must be 1+, not {0:n}".format(
                chunksize))
    task_batches = mpp.Pool._get_tasks(func, iterable, chunksize)
    result = mpp.IMapIterator(self._cache)
    self._taskqueue.put(
        (
            self._guarded_task_generation(result._job,
                                          mpp.starmapstar,
                                          task_batches),
            result._set_length
        ))
    return (item for chunk in result for item in chunk)


def istarmap38(self, func, iterable, chunksize=1):
    """starmap-version of imap
    """
    self._check_running()
    if chunksize < 1:
        raise ValueError(
            "Chunksize must be 1+, not {0:n}".format(
                chunksize))

    task_batches = mpp.Pool._get_tasks(func, iterable, chunksize)
    result = mpp.IMapIterator(self)
    self._taskqueue.put(
        (
            self._guarded_task_generation(result._job,
                                          mpp.starmapstar,
                                          task_batches),
            result._set_length
        ))
    return (item for chunk in result for item in chunk)


if sys.version_info.major == 3 and sys.version_info.minor >= 8:
    mpp.Pool.istarmap = istarmap38
else:
    mpp.Pool.istarmap = istarmap37


def ignore_no_use_warning():
    pass
